SystemOrganization addCategory: #GODSpreadsheet!

GODData subclass: #SSSpreadsheetData
	instanceVariableNames: 'sheets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSpreadsheet'!

!SSSpreadsheetData commentStamp: 'jjs 10/18/2014 17:56' prior: 0!
Creates a spreadsheet with a set of sheets!

!SSSpreadsheetData class methodsFor: 'private' stamp: 'foa 10/18/2014 16:13'!
fromCSV: filePath
	"initialize a SSSpreadsheetData from specified CSV"
	|file line row columns ssdata sheet count|
	ssdata := SSSpreadsheetData new.
	sheet := ssdata createSheetWithName: 'default'.
	
	file := FileStream fileNamed: filePath.
	[file atEnd] whileFalse:
		[line := file nextLine.
		columns := line findTokens: ',' escapedBy: '"'.
		row := sheet createRow.
		count := 1.
		columns do: [:item | row createCellAtColumnIndex: count withValue: item. count := count + 1]
		].
	^ssdata! !

!SSSpreadsheetData class methodsFor: 'initializing' stamp: 'dhk 10/31/2014 23:27'!
fromCollection: aCollection
	"initialize a SSSpreadsheetData with a single SSSheet and fills it with aCollection"
	|data sheet|
	data := SSSpreadsheetData new.
	sheet := data createSheetWithName: 'default'.
	sheet fillFromCollection: aCollection.
	
	^data
	! !

!SSSpreadsheetData class methodsFor: 'initializing' stamp: 'foa 10/31/2014 23:20'!
fromDictionary: aDictionary
	"initialize a SSSpreadsheetData with a single SSSheet and fills it with aDictionary"
	
	| ssdata sheet | 
	ssdata := SSSpreadsheetData new.
	sheet := ssdata createSheetWithName: 'default'.
	sheet fillFromDictionary: aDictionary.
	
	^ssdata! !

!SSSpreadsheetData class methodsFor: 'initializing' stamp: 'foa 11/1/2014 18:18'!
fromFile: filePath
	"initialize a SSSpreadsheetData from specified path"
	|extension|
	extension := (filePath subStrings: '.') last asUppercase.
	extension = 'CSV'
		ifTrue: [^self fromCSV: filePath].
	extension = 'ODS'
		ifTrue: [^self fromODS: filePath].
	extension = 'XLSX'
		ifTrue: [^self fromXLSX: filePath].! !

!SSSpreadsheetData class methodsFor: 'private' stamp: 'foa 10/18/2014 18:16'!
fromODS: filePath
	"initialize a SSSpreadsheetData from specified ODS"
	
	| odsFile contentsXML data root ssdata sheet row colIndex rowsRepeat columnsRepeat |
	
	odsFile := ZipArchive new readFrom: filePath.
	contentsXML := odsFile member: 'content.xml'.
	data := XMLDOMParser parseDocumentFrom: contentsXML contents readStream.
	root := data elementAt: #'office:document-content'.
	
	ssdata := SSSpreadsheetData new.
	root tagsNamed: #'table:table' do: [ :table |
		sheet := ssdata createSheetWithName: (table attributeAt: 'table:name').
		table tagsNamed: #'table:table-row' do: [ :trow |
			rowsRepeat := trow attributeAt: #'table:number-rows-repeated'.
			rowsRepeat isNil ifTrue: [ rowsRepeat := 1 ] ifFalse: [ rowsRepeat := rowsRepeat asNumber ].
			rowsRepeat timesRepeat: [
				row := sheet createRow.
				colIndex := 1.
				trow tagsNamed: #'table:table-cell' do: [ :cell |
					columnsRepeat := cell attributeAt: #'table:number-columns-repeated'.
					columnsRepeat isNil ifTrue: [ columnsRepeat := 1 ] ifFalse: [ columnsRepeat := columnsRepeat asNumber ].
					columnsRepeat timesRepeat: [
						row createCellAtColumnIndex: colIndex withValue: (SSSpreadsheetData getODSCellValue: cell).
						colIndex := colIndex + 1.
						].
					].
				].
			].
		].
	^ssdata.! !

!SSSpreadsheetData class methodsFor: 'private' stamp: 'foa 11/1/2014 20:56'!
fromXLSX: filePath
	"initialize a SSSpreadsheetData from specified XLSX"

	| xlsxFile stringsXML stringsRoot sharedStrings workbookXML workbookRoot ssdata sheet sheetXML sheetRoot row | 
	xlsxFile := ZipArchive new readFrom: filePath.
	
	"1) store the strings in a ordered collection"
	stringsXML := xlsxFile member: 'xl/sharedStrings.xml'.
	stringsRoot := (XMLDOMParser parseDocumentFrom: stringsXML contents readStream) elementAt: 'sst'.
	sharedStrings := OrderedCollection new.
	stringsRoot tagsNamed: #'t' do: [ :t |
		sharedStrings add: (t elementsAndContents first characterData).
		].
		
	"2) get the name and number of sheets in the file"
	workbookXML := xlsxFile member: 'xl/workbook.xml'.
	workbookRoot := (XMLDOMParser parseDocumentFrom: workbookXML contents readStream) elementAt: 'workbook'.
	ssdata := SSSpreadsheetData new.
	workbookRoot tagsNamed: #'sheet' do: [ :sheetTag |
		sheet := ssdata createSheetWithName: (sheetTag attributeAt: 'name').
		
		"3) for each sheet, load its contents"
		sheetXML := xlsxFile member: ('xl/worksheets/sheet' , (sheetTag attributeAt: 'sheetId') , '.xml').
		sheetRoot := (XMLDOMParser parseDocumentFrom: sheetXML contents readStream) elementAt: 'worksheet'.
		
		"3.A) first, iterate through rows"
		sheetRoot tagsNamed: #'row' do: [ :rowTag |
			row := sheet createRowAtIndex: ((rowTag attributeAt: 'r') asNumber).
			
			"3.B) then, iterate over the cells in this row"
			rowTag tagsNamed: #'c' do: [ :cellTag |
				row createCellAtColumn: ((cellTag attributeAt: 'r') onlyLetters) withValue: (SSSpreadsheetData getXLSXCellValue: cellTag withStrings: sharedStrings).
				].
			].
		].
	
	^ssdata! !

!SSSpreadsheetData class methodsFor: 'utility' stamp: 'foa 11/1/2014 21:23'!
getDateFromXLSXValue: value
	"Converts the date value from a XLSX spreadsheet, which is the number of days since epoch, to its equivalent string representation day/month/year."

	^value
	! !

!SSSpreadsheetData class methodsFor: 'utility' stamp: 'foa 11/1/2014 19:56'!
getODSCellValue: aCell
	"Gets the value of the given cell (from a ODS spreadsheet)"

	| internalData |
	internalData := aCell firstTagNamed: #'text:p'.
	internalData isNil ifTrue: [ ^'' ].
	
	^internalData elementsAndContents first characterData.! !

!SSSpreadsheetData class methodsFor: 'utility' stamp: 'foa 11/1/2014 21:30'!
getTimeFromXLSXValue: value
	"Converts the time value from a XLSX spreadsheet, which is a percentage between 0 and 1, to its equivalent string representation hour:minute of the day."

	| time hours minutes minStr |
	time := 24 * value.
	hours := time asInteger.
	minutes := ((time fractionPart) * 60) rounded.
	minStr := minutes asString.
	minStr size = 1 ifTrue: [ minStr := '0' , minStr ].
	^(hours asString) , ':' , minStr
	! !

!SSSpreadsheetData class methodsFor: 'utility' stamp: 'foa 11/1/2014 21:27'!
getXLSXCellValue: cellTag withStrings: sharedStrings
	"Gets the value of the given cell tag (from a XLSX spreadsheet), with the given shared strings collection."

	| internalValue type subType date time | 
	internalValue := (cellTag firstTagNamed: #'v') elementsAndContents first characterData.
	
 	type := cellTag attributeAt: #'t'.
	type isNil 
		ifTrue: [
			subType := cellTag attributeAt: #'s'.
			subType isNil ifFalse: [
				subType := subType asNumber.
				subType = 1 ifTrue: [ "Date value"
					internalValue := SSSpreadsheetData getDateFromXLSXValue: (internalValue asNumber).
					].
				subType = 2 ifTrue: [ "Time value"
					internalValue := SSSpreadsheetData getTimeFromXLSXValue: (internalValue asNumber).
					].
				"subType=3 should be Percentage, but in this case we'll store the same value (1.2, as stored in the XLSX, instead of 120%)"
				subType = 4 ifTrue: [ "Date+time value"
					internalValue := internalValue asNumber.
					date := SSSpreadsheetData getDateFromXLSXValue: (internalValue integerPart).
					time := SSSpreadsheetData getTimeFromXLSXValue: (internalValue fractionPart).
					internalValue := time , ' ' , date.
					].
				].
			^internalValue
			]
		ifFalse: [
			type = 's' ifTrue: [
				"This cell is a string in the shared strings collection, indexed by internalValue."
				^sharedStrings at: ((internalValue asNumber) + 1).
				].
			].
	^internalValue! !

!SSSpreadsheetData methodsFor: 'adding' stamp: 'dhk 10/29/2014 22:16'!
createSheetWithName: name
	"creates a sheet with name 'name'"
	|sheet|
	sheet := SSSheet new: name.
	sheet parent: self.
	sheets add: sheet.
	sheet index: sheets size.
	^sheet! !

!SSSpreadsheetData methodsFor: 'accessing' stamp: 'dhk 10/17/2014 18:17'!
getSheet: sheetNumber
	"returns the sheet with the sheetNumber index"
	
	^sheets at: sheetNumber.! !

!SSSpreadsheetData methodsFor: 'accessing' stamp: 'dhk 10/17/2014 18:23'!
getSheetWithName: sheetName
	"returns the sheet with name sheetName"
	sheets do:
		[:sheet | (sheet name = sheetName) ifTrue: [^sheet]].
	^nil! !

!SSSpreadsheetData methodsFor: 'initialize' stamp: 'jjs 10/18/2014 17:55'!
initialize
	"creates a new empty set of sheets and returns the set of sheets"
	sheets := OrderedCollection new.
	^self! !

!SSSpreadsheetData methodsFor: 'accessing' stamp: 'dhk 9/27/2014 17:23'!
numberOfSheets
	"returns how many sheets exists in this GODSpreadsheetData"

	^sheets size! !

!SSSpreadsheetData methodsFor: 'accessing' stamp: 'jjs 10/18/2014 17:39'!
renameSheet: sheetName toName: aName
	"renames a sheet with name sheetName to the name aName"
	|sheet|
	sheet := (self getSheetWithName: sheetName).
	sheet changeName: aName.! !

!SSSpreadsheetData methodsFor: 'file out' stamp: 'dhk 10/2/2014 21:51'!
toFile: filePath
	"save this SSSpreadsheetData to specified path. Returns false if there is an error"

	^true! !

Object subclass: #SSCell
	instanceVariableNames: 'value parent column'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSpreadsheet'!

!SSCell commentStamp: 'jjs 10/18/2014 17:46' prior: 0!
creates an cell object with a parent row, column index and cell's value!

!SSCell class methodsFor: 'initialize' stamp: 'dhk 10/17/2014 16:45'!
newWithParent: aParent atColumn: aColumnName

	^((self new) parent: aParent) column: aColumnName.! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/26/2014 15:44'!
column
	"Return the column name of this cell.
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."

	^column! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/29/2014 17:51'!
column: aColumn
	"Set the column name of this cell.
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."

	column := (aColumn asUppercase).! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:22'!
getColumnIndex
	"Returns the column index of this cell."

	^SSSheet columnNameToIndex: column! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:47'!
getParentRow
	"Returns the Row which this cell belongs to."

	^self parent.! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:23'!
getRowIndex
	"Returns the row index of this cell. "

	^parent index.! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:47'!
parent
	"Returns the Row which this cell belongs to."

	^parent.! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:06'!
parent: aParent
	"set the parent of this cell"

	parent := aParent! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/2/2014 21:40'!
value
	"returns the value of this cell"

	^value! !

!SSCell methodsFor: 'accessing' stamp: 'dhk 10/2/2014 21:40'!
value: aValue
	"set the value of this cell"

	value := aValue! !

Object subclass: #SSRow
	instanceVariableNames: 'cells parent index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSpreadsheet'!

!SSRow commentStamp: 'jjs 10/18/2014 17:49' prior: 0!
Creates and row object with a set of cells, parent sheet and row index!

!SSRow methodsFor: 'adding' stamp: 'dhk 10/26/2014 15:46'!
addCell: aCell atColumn: columnName
	"add aCell at the column with name columnName
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."
	self addCell: aCell atColumnIndex: (SSSheet columnNameToIndex: columnName).
	
	
	
	! !

!SSRow methodsFor: 'adding' stamp: 'dhk 10/26/2014 15:46'!
addCell: aCell atColumnIndex: columnIndex
	"add a cell in this row at the specified column index (an integer)"
	[((cells size)) < columnIndex]
		whileTrue:
		[cells add: (SSCell newWithParent: self atColumn: (SSSheet columnIndexToName: cells size + 1))].
	cells at: columnIndex put: aCell.! !

!SSRow methodsFor: 'deleting' stamp: 'dhk 11/1/2014 10:50'!
clearCell: columnName
	"clear the content of this cell at the column with name columnName
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."
	self clearCellAtIndex: (SSSheet columnNameToIndex: columnName).! !

!SSRow methodsFor: 'deleting' stamp: 'dhk 11/1/2014 10:48'!
clearCellAtIndex: columnIndex
	"clear the content of this cell at specified column index (an integer)"
	(cells at: columnIndex) value: nil.! !

!SSRow methodsFor: 'adding' stamp: 'dhk 10/26/2014 15:46'!
createCellAtColumn: columnName withValue: aValue
	"create cell at the column with name columnName and value aValue
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."
	|aCell|
	aCell := (SSCell newWithParent: self atColumn: columnName) value: aValue.
	self addCell: aCell atColumn: columnName.
	^aCell
	! !

!SSRow methodsFor: 'adding' stamp: 'dhk 10/26/2014 15:46'!
createCellAtColumnIndex: columnIndex withValue: aValue
	"create cell at the column index columnIndex (an integer) with value aValue"
	^self createCellAtColumn: (SSSheet columnIndexToName: columnIndex) withValue: aValue
	
	! !

!SSRow methodsFor: 'accessing' stamp: 'dhk 11/1/2014 00:10'!
do: aBlock
"do aBlock for each cell in this row"

cells do: aBlock.! !

!SSRow methodsFor: 'accessing' stamp: 'dhk 10/26/2014 15:45'!
getCell: columnName
	"get the cell in this row at the column with name columnName
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."

	^self getCellAtIndex: (SSSheet columnNameToIndex: columnName).! !

!SSRow methodsFor: 'accessing' stamp: 'dhk 10/26/2014 15:45'!
getCellAtIndex: columnIndex
	"get the cell in this row at columnIndex (an integer)"

	^cells at: columnIndex.! !

!SSRow methodsFor: 'accessing' stamp: 'dhk 10/26/2014 15:45'!
index
	"Returns the index (an integer) of this row in the parent sheet."

	^index.! !

!SSRow methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:16'!
index: aIndex
	"set the index of this cell"
	index := aIndex! !

!SSRow methodsFor: 'initialize' stamp: 'jjs 10/18/2014 17:28'!
initialize
      "initialize this instance with an empty cell collection"
	cells := OrderedCollection new.! !

!SSRow methodsFor: 'accessing' stamp: 'dhk 10/17/2014 16:52'!
parent
	"Returns the sheet which this row belongs to."

	^parent.! !

!SSRow methodsFor: 'accessing' stamp: 'jjs 10/18/2014 17:28'!
parent: aParent
     "Sets the sheet which this row belongs to."

	parent := aParent.! !

Object subclass: #SSSheet
	instanceVariableNames: 'sheetName rows parent index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GODSpreadsheet'!

!SSSheet commentStamp: 'jjs 10/18/2014 17:51' prior: 0!
Creates an spreadsheet object with a spreadsheetData parent, a sheet name, a set of rows and a sheet index !

!SSSheet class methodsFor: 'utility' stamp: 'foa 10/17/2014 17:51'!
columnIndexToName: aIndex
	"Utility method to convert a column index value to a column name."
	
	| rest subIndex |
	aIndex <= 26
		ifTrue:	[^(aIndex+64) asCharacter asString].
	rest := (aIndex - 1) \\ 26.
	subIndex := (aIndex - 1) // 26.
	^(self columnIndexToName: subIndex), ((rest + 65) asCharacter asString)! !

!SSSheet class methodsFor: 'utility' stamp: 'dhk 10/29/2014 17:50'!
columnNameToIndex: columnName
	"Utility method to convert a column name value to a column index."
     "index = 0
	l = list(name)
	index = ord(l[0]) - 64
	for i in xrange(1, len(l)):
		index *= 26
		index += ord(l[i]) - 64
	return index"
	
	| index aName | 
	aName := (columnName asUppercase).
	index := (aName first asInteger) - 64.
	aName allButFirstDo: [ :char |
		index := (index * 26) + ((char asInteger) - 64)].
	^index! !

!SSSheet class methodsFor: 'instance creation' stamp: 'dhk 9/27/2014 16:35'!
new: aName
	^(self new) changeName: aName! !

!SSSheet methodsFor: 'private' stamp: 'dhk 9/27/2014 16:39'!
changeName: aName
	"change the name of this sheet"

	sheetName := aName! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/29/2014 18:05'!
createCellAtRow: rowIndex atColumn: columnName withValue: aValue
	"creates a cell at this specified address with value aValue and returns it
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."
	| row cell |
	rows size < rowIndex
		ifTrue: [row := (self createRowAtIndex: rowIndex)]
		ifFalse:[row := rows at: rowIndex].
	cell := row createCellAtColumn: columnName withValue: aValue.
	^cell! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/29/2014 18:07'!
createCellAtRow: rowIndex atColumnIndex: columnIndex withValue: aValue
	"creates a cell at this specified address with value aValue and returns it"
	^self createCellAtRow: rowIndex atColumn: (SSSheet columnIndexToName: columnIndex) withValue: aValue.! !

!SSSheet methodsFor: 'adding' stamp: 'jjs 10/18/2014 17:32'!
createRow
	"creates a new empty row after the last row in this sheet and returns it"
	|row|
	row := ((SSRow new) parent: self).
	rows add: row.
	row index: (rows size).
	^row.! !

!SSSheet methodsFor: 'adding' stamp: 'dhk 10/29/2014 22:10'!
createRowAtIndex: aIndex
	"creates a new empty row at a specified index in this sheet.
	This method creates empty rows if needed"
	
	[((rows size) + 1) < aIndex]
		whileTrue:
			[self createRow].
	^self createRow! !

!SSSheet methodsFor: 'deleting' stamp: 'dhk 10/17/2014 19:28'!
deleteRow: aIndex
	"delete row at index aIndex"
	
	rows removeAt: aIndex.! !

!SSSheet methodsFor: 'file handling' stamp: 'dhk 11/1/2014 00:29'!
exportToCSV: filePath
	"Export this sheet to a csv file. Cells containing comma ( , ) will be escaped with double quotes"
	| file |
	file := FileStream fileNamed: filePath.
	rows do: [:row |
		row do: [ :cell |
			(cell value includesSubString: ',')
			ifTrue: 
				[file nextPut: $"; nextPutAll: cell value; nextPut: $";nextPut: $,. ]
			ifFalse:
				[file nextPutAll: cell value; nextPut: $,. ].
			].
		file position: file position -1; cr.
		].

	file close.! !

!SSSheet methodsFor: 'initialize' stamp: 'dhk 10/31/2014 23:29'!
fillFromCollection: aCollection
"Fill this sheet with values from aCollection.
aCollection must be a collection of collections"
| row colIndex |
aCollection do:  [ :lineC |
	row := self createRow.
	colIndex := 1.
	lineC do: [ :item |
		row createCellAtColumnIndex: colIndex withValue: item.
		colIndex := colIndex + 1.
		].
	].! !

!SSSheet methodsFor: 'initialize' stamp: 'foa 10/31/2014 23:17'!
fillFromDictionary: aDictionary
"Fill this sheet with values from aDictionary. The dictionary must be a simple one, without collections. Each key-value pair will be a row,
 with the key being in column A and value being in column B."

| row | 
aDictionary keysDo: [ :key |
	row := self createRow.
	row createCellAtColumn: 'A' withValue: key.
	row createCellAtColumn: 'B' withValue: (aDictionary at: key).
	]! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/26/2014 15:48'!
getCellAtRow: rowIndex atColumn: columnName
	"returns the cell at this specified address
	We expect column names as used in common office spreadsheet editing applications. Columns start from A to Z, then AA to AZ, BA and so on..."

	^self getCellAtRow: rowIndex atColumnIndex: (SSSheet columnNameToIndex: columnName).! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/17/2014 17:22'!
getCellAtRow: rowIndex atColumnIndex: columnIndex
	"returns the cell at this specified address"

	^(rows at: rowIndex) getCellAtIndex: columnIndex.! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/17/2014 17:20'!
getRow: rowIndex
	"returns the row with the specified index"

	^rows at: rowIndex! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/29/2014 22:16'!
index
	"return the index of this sheet in the SpreadsheetData"

	^index.! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/29/2014 22:16'!
index: aIndex
	"set the index of this sheet in the SpreadsheetData"

	index := aIndex.! !

!SSSheet methodsFor: 'initialize' stamp: 'dhk 10/17/2014 17:18'!
initialize
	rows := OrderedCollection new.! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/2/2014 21:39'!
name
	"returns the name of this sheet"

	^sheetName! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/17/2014 17:19'!
parent
	"return the SSSpreadsheetData which this sheet belongs to"

	^parent! !

!SSSheet methodsFor: 'accessing' stamp: 'dhk 10/17/2014 17:19'!
parent: aParent
	"set the SSSpreadsheetData which this sheet belongs to"

	parent := aParent! !

!SSSheet methodsFor: 'sorting' stamp: 'dhk 11/1/2014 12:13'!
sort
	"sort (using in-place quick sort) all rows by column A"	
	^self sortFrom: 1.! !

!SSSheet methodsFor: 'sorting' stamp: 'dhk 11/1/2014 12:06'!
sortFrom: fromIndex
	"sort rows (using in-place quick sort) from fromIndex to the last row by column A"
	^self sortFrom: fromIndex to: rows size.! !

!SSSheet methodsFor: 'sorting' stamp: 'dhk 11/1/2014 12:06'!
sortFrom: fromIndex to: toIndex
	"sort rows (using in-place quick sort) from fromIndex to toIndex by column A"
	^self sortFrom: fromIndex to: toIndex by: 
	[:r1 :r2 | (r1 getCell: 'A') isNil ifTrue: [false] 
			ifFalse: [ (r2 getCell: 'A') isNil ifTrue: [true]
				ifFalse: [ (r1 getCell: 'A') value isNil ifTrue: [false]
					ifFalse: [ (r2 getCell: 'A') value isNil ifTrue: [true]
						ifFalse: [ (r1 getCell: 'A') value < (r2 getCell: 'A') value ]
						]
					]
				]
			]! !

!SSSheet methodsFor: 'sorting' stamp: 'dhk 11/1/2014 12:06'!
sortFrom: fromIndex to: toIndex by: sortBlock
	"sort rows (using in-place quick sort) from fromIndex to toIndex according to sortBlock"
	rows := OrderedCollection newFrom: ((rows asArray) quickSortFrom: fromIndex to: toIndex by: sortBlock)! !
